@@ -11,47 +11,46 @@ Usage:
   liquidctl --version
 
 Device selection options (see: list -v):
-  -m, --match <substring>     Filter devices by description substring
-  -n, --pick <number>         Pick among many results for a given filter
-  --vendor <id>               Filter devices by hexadecimal vendor ID
-  --product <id>              Filter devices by hexadecimal product ID
-  --release <number>          Filter devices by hexadecimal release number
-  --serial <number>           Filter devices by serial number
-  --bus <bus>                 Filter devices by bus
-  --address <address>         Filter devices by address in bus
-  --usb-port <port>           Filter devices by USB port in bus
-  -d, --device <id>           Select device by listing id
+  -m, --match <substring>        Filter devices by description substring
+  -n, --pick <number>            Pick among many results for a given filter
+  --vendor <id>                  Filter devices by hexadecimal vendor ID
+  --product <id>                 Filter devices by hexadecimal product ID
+  --release <number>             Filter devices by hexadecimal release number
+  --serial <number>              Filter devices by serial number
+  --bus <bus>                    Filter devices by bus
+  --address <address>            Filter devices by address in bus
+  --usb-port <port>              Filter devices by USB port in bus
+  -d, --device <id>              Select device by listing id
 
 Animation options (devices/modes can support zero or more):
-  --speed <value>             Abstract animation speed (device/mode specific)
-  --time-per-color <value>    Time to wait on each color (seconds)
-  --time-off <value>          Time to wait with the LED turned off (seconds)
-  --alert-threshold <number>  Threshold temperature for a visual alert (°C)
-  --alert-color <color>       Color used by the visual high temperature alert
+  --speed <value>                Abstract animation speed (device/mode specific)
+  --time-per-color <value>       Time to wait on each color (seconds)
+  --time-off <value>             Time to wait with the LED turned off (seconds)
+  --alert-threshold <number>     Threshold temperature for a visual alert (°C)
+  --alert-color <color>          Color used by the visual high temperature alert
+  --direction <string>           If the pattern should move forward or backwards. [default: forward]
+  --start-led <number>           The first led to start the effect at
+  --maximum-leds <number>        The number of LED's the effect should apply to
 
 Other device options:
-  --single-12v-ocp            Enable single rail +12V OCP
-  --pump-mode <mode>          Set the pump mode (certain Corsair coolers)
-  --legacy-690lc              Use Asetek 690LC in legacy mode (old Krakens)
-  --unsafe <features>         Comma-separated bleeding-edge features to enable
+  --single-12v-ocp               Enable single rail +12V OCP
+  --pump-mode <mode>             Set the pump mode (certain Corsair coolers)
+  --temperature-sensor <number>  The temperature sensor number for the Commander Pro
+  --legacy-690lc                 Use Asetek 690LC in legacy mode (old Krakens)
+  --non-volatile                 Store on non-volatile controller memory
+  --unsafe <features>            Comma-separated bleeding-edge features to enable
 
 Other interface options:
-  -v, --verbose               Output additional information
-  -g, --debug                 Show debug information on stderr
-  --version                   Display the version number
-  --help                      Show this message
+  -v, --verbose                  Output additional information
+  -g, --debug                    Show debug information on stderr
+  --version                      Display the version number
+  --help                         Show this message
 
 Deprecated:
-  --hid <ignored>             Deprecated
+  --hid <ignored>                Deprecated
 
-Examples:
-  liquidctl list --verbose
-  liquidctl initialize all
-  liquidctl --match kraken set pump speed 90
-  liquidctl --product 170e set led color fading 350017 ff2608
-  liquidctl status
-
-Copyright (C) 2018–2020  Jonas Malaco, CaseySJ, Tom Frey and contributors
+Copyright (C) 2018–2021  Jonas Malaco, Marshall Asch, CaseySJ, Tom Frey and
+contributors
 
 liquidctl incorporates work by leaty, Ksenija Stanojevic, Alexander Tong, Jens
 Neumaier, Kristóf Jakab, Sean Nelson, Chris Griffith, notaz, realies and Thomas
@@ -69,11 +68,12 @@ import inspect
 import logging
 import os
 import sys
+import errno
 
 from docopt import docopt
 
 from liquidctl.driver import *
-from liquidctl.error import NotSupportedByDevice, NotSupportedByDriver
+from liquidctl.error import NotSupportedByDevice, NotSupportedByDriver, UnsafeFeaturesNotEnabled
 from liquidctl.util import color_from_str
 from liquidctl.version import __version__
 
@@ -98,10 +98,15 @@ _PARSE_ARG = {
     '--time-off': int,
     '--alert-threshold': int,
     '--alert-color': color_from_str,
+    '--temperature-sensor': int,
+    '--direction': str,
+    '--start-led': int,
+    '--maximum-leds': int,
 
     '--single-12v-ocp': bool,
     '--pump-mode': str,
     '--legacy-690lc': bool,
+    '--non-volatile': bool,
     '--unsafe': lambda x: x.lower().split(','),
     '--verbose': bool,
     '--debug': bool,
@@ -122,14 +127,14 @@ _FILTER_OPTIONS = [
 
 # custom number formats for values of select units
 _VALUE_FORMATS = {
-    '°C' : '.1f',
-    'rpm' : '.0f',
-    'V' : '.2f',
-    'A' : '.2f',
-    'W' : '.2f'
+    '°C': '.1f',
+    'rpm': '.0f',
+    'V': '.2f',
+    'A': '.2f',
+    'W': '.2f'
 }
 
-LOGGER = logging.getLogger(__name__)
+_LOGGER = logging.getLogger(__name__)
 
 
 def _list_devices(devices, using_filters=False, device_id=None, verbose=False, debug=False, **opts):
@@ -145,9 +150,12 @@ def _list_devices(devices, using_filters=False, device_id=None, verbose=False, d
         if not verbose:
             continue
 
-        print(f'├── Vendor ID: {dev.vendor_id:#06x}')
-        print(f'├── Product ID: {dev.product_id:#06x}')
-        print(f'├── Release number: {dev.release_number:#06x}')
+        if dev.vendor_id:
+            print(f'├── Vendor ID: {dev.vendor_id:#06x}')
+        if dev.product_id:
+            print(f'├── Product ID: {dev.product_id:#06x}')
+        if dev.release_number:
+            print(f'├── Release number: {dev.release_number:#06x}')
         try:
             if dev.serial_number:
                 print(f'├── Serial number: {dev.serial_number}')
@@ -158,7 +166,7 @@ def _list_devices(devices, using_filters=False, device_id=None, verbose=False, d
             elif sys.platform in ['win32', 'cygwin'] and 'Hid' not in type(dev.device).__name__:
                 msg += ' (device possibly requires a kernel driver)'
             if debug:
-                LOGGER.exception(msg.capitalize())
+                _LOGGER.exception(msg.capitalize())
             else:
                 warnings.append(msg)
 
@@ -168,22 +176,22 @@ def _list_devices(devices, using_filters=False, device_id=None, verbose=False, d
             port = '.'.join(map(str, dev.port))
             print(f'├── Port: {port}')
 
-        print(f'└── Driver: {type(dev).__name__} using module {dev.device.api.__name__}')
+        print(f'└── Driver: {type(dev).__name__}')
         if debug:
             driver_hier = [i.__name__ for i in inspect.getmro(type(dev)) if i != object]
-            LOGGER.debug('hierarchy: %s; %s', ', '.join(driver_hier[1:]), type(dev.device).__name__)
+            _LOGGER.debug('hierarchy: %s', ', '.join(driver_hier[1:]))
 
         for msg in warnings:
-            LOGGER.warning(msg)
+            _LOGGER.warning(msg)
         print('')
 
-    assert not 'device' in opts or len(devices) <= 1, 'too many results listed with --device'
+    assert 'device' not in opts or len(devices) <= 1, 'too many results listed with --device'
 
 
 def _print_dev_status(dev, status):
     if not status:
         return
-    print(f'{dev.description}')
+    print(dev.description)
     tmp = []
     kcols, vcols = 0, 0
     for k, v, u in status:
@@ -218,8 +226,8 @@ def _device_set_speed(dev, args, **opts):
 
 def _make_opts(args):
     if args['--hid']:
-        LOGGER.warning('Ignoring --hid %s: deprecated option, API will be selected automatically',
-                       args['--hid'])
+        _LOGGER.warning('ignoring --hid %s: deprecated option, API will be selected automatically',
+                        args['--hid'])
     opts = {}
     for arg, val in args.items():
         if val is not None and arg in _PARSE_ARG:
@@ -241,8 +249,8 @@ def _gen_version():
             if __extraversion__['dirty']:
                 extra[0] += '-dirty'
     except:
-        return 'liquidctl v{}'.format(__version__)
-    return 'liquidctl v{} ({})'.format(__version__, '; '.join(extra))
+        return f'liquidctl v{__version__}'
+    return f'liquidctl v{__version__} ({"; ".join(extra)})'
 
 
 def main():
@@ -255,7 +263,7 @@ def main():
     if args['--debug']:
         args['--verbose'] = True
         logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] %(name)s: %(message)s')
-        LOGGER.debug('running %s', _gen_version())
+        _LOGGER.debug('running %s', _gen_version())
     elif args['--verbose']:
         logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s')
     else:
@@ -279,8 +287,8 @@ def main():
             matched_devs = [dev.device for dev in find_liquidctl_devices(**opts)]
             if compat[device_id].device not in matched_devs:
                 raise SystemExit('Error: device ID does not match remaining selection criteria')
-            LOGGER.warning('mixing --device <id> with other filters is not recommended; '
-                           'to disambiguate between results prefer --pick <result>')
+            _LOGGER.warning('mixing --device <id> with other filters is not recommended; '
+                            'to disambiguate between results prefer --pick <result>')
         selected = [compat[device_id]]
 
     if args['list']:
@@ -292,10 +300,18 @@ def main():
     elif len(selected) == 0:
         raise SystemExit('Error: no devices matches available drivers and selection criteria')
 
+    errors = 0
+
+    def log_error(err, msg, *args):
+        nonlocal errors
+        errors += 1
+        _LOGGER.info('%s', err, exc_info=True)
+        _LOGGER.error(msg, *args)
+
     for dev in selected:
-        LOGGER.debug('device: %s', dev.description)
-        dev.connect(**opts)
+        _LOGGER.debug('device: %s', dev.description)
         try:
+            dev.connect(**opts)
             if args['initialize']:
                 _print_dev_status(dev, dev.initialize(**opts))
             elif args['status']:
@@ -305,21 +321,36 @@ def main():
             elif args['set'] and args['color']:
                 _device_set_color(dev, args, **opts)
             else:
-                raise Exception('Not sure what to do')
-        except NotSupportedByDevice:
-            raise SystemExit(f'Error: operation not supported by {dev.description}')
-        except NotSupportedByDriver:
-            raise SystemExit(f'Error: operation not supported by driver for {dev.description}')
-        except:
-            LOGGER.exception('Unexpected error with %s', dev.description)
-            sys.exit(1)
+                raise Exception('not sure what to do')
+        except OSError as err:
+            # each backend API returns a different subtype of OSError (OSError,
+            # usb.core.USBError or PermissionError) for permission issues
+            if err.errno in [errno.EACCES, errno.EPERM]:
+                log_error(err, f'Error: insufficient permissions to access {dev.description}')
+            elif err.args == ('open failed', ):
+                log_error(err, f'Error: could not open {dev.description}, possibly due to insufficient permissions')
+            else:
+                log_error(err, f'Unexpected OS error with {dev.description}: {err}')
+        except NotSupportedByDevice as err:
+            log_error(err, f'Error: operation not supported by {dev.description}')
+        except NotSupportedByDriver as err:
+            log_error(err, f'Error: operation not supported by driver for {dev.description}')
+        except UnsafeFeaturesNotEnabled as err:
+            features = ','.join(err.args)
+            log_error(err, f'Error: missing --unsafe features for {dev.description}: {features!r}')
+            _LOGGER.error('More information is provided in the corresponding device guide')
+        except Exception as err:
+            log_error(err, f'Unexpected error with {dev.description}: {err}')
         finally:
             dev.disconnect(**opts)
 
+    if errors:
+        sys.exit(errors)
+
 
 def find_all_supported_devices(**opts):
     """Deprecated."""
-    LOGGER.warning('deprecated: use liquidctl.driver.find_liquidctl_devices instead')
+    _LOGGER.warning('deprecated: use liquidctl.driver.find_liquidctl_devices instead')
     return find_liquidctl_devices(**opts)
 
 
